`default_nettype none
module iirhp(y, rdy, x, nd, dither, freqsel, firclock, reset);
// Stereo IIR high-pass, processes left sample, right sample, left sample and
// so on.
// freqsel = 2'b00: integrators frozen, 0 Hz cut-off
// freqsel = 2'b01: 1.211668 Hz cut-off at 88.2 kHz sample rate
// freqsel = 2'b10: 4.846672 Hz cut-off at 88.2 kHz sample rate
// freqsel = 2'b11: 9.693344 Hz cut-off at 88.2 kHz sample rate
// In all cases, the response is close to second-order Butterworth.
input wire nd, firclock, reset;
input wire signed [62:0] x;
input wire signed [64:0] dither; // more bits than required, superfluous bits won't be connected
input wire [1:0] freqsel;
output reg signed [26:0] y;
output reg rdy;
reg [4:0] ndshift;
reg oldnd;
reg signed [38:0] i1;
reg signed [38:0] i1t;
reg signed [52:0] i2;
reg signed [52:0] i2t;
reg signed [64:0] a;
always @ (posedge firclock)
begin
	if (reset)
	begin
		ndshift[4:0] <= 5'b0000;
		oldnd <= 1'b0;
		i1 <= 39'sb0;
		i1t <= 39'sb0;
		i2 <= 53'sb0;
		i2t <= 53'sb0;
		a <= 65'sb0; // 249 extra multiplexers zo, kan dat efficienter?
		rdy <= 1'b0;
		y <= 27'sb0;
	end
	else
	begin
		ndshift[4:1] <= ndshift[3:0];
		ndshift[0] <= nd && (!oldnd); // nodig als nd toch maar 1 klokpuls breed is? Denk het niet.
		oldnd <= nd;
		case (ndshift[4:0])
			5'b00001:
			begin
				a <= x + $signed(dither[64:26]); // ## Klopt die 25? Ik denk het wel.
				rdy <= 1'b0;
			end
			5'b00010:
			begin
				a <= a - (i2 <<< 11);
				rdy <= 1'b0;
			end
			5'b00100:
			begin
				a <= a - (i1 <<< 25);
				rdy <= 1'b0;
			end
			5'b01000:
			begin
				y[26:0] <= a[64:38];
				rdy <= 1'b1;
				case (freqsel)
					2'b00:
					begin
						i2t <= i2;
						i1t <= i1;
					end
					2'b01:
					begin
						i2t <= i2 + i1;
						i1t <= i1 + $signed(a[64:38]);
					end
					2'b10:
					begin
						i2t <= i2 + (i1 <<< 2);
						i1t <= i1 + ($signed(a[64:38]) <<< 2);
					end
					2'b11:
					begin
						i2t <= i2 + (i1 <<< 3);
						i1t <= i1 + ($signed(a[64:38]) <<< 3);
					end
				endcase;
				i2 <= i2t;
				i1 <= i1t;
			end
			5'b10000: rdy <= 1'b0;
		endcase;
	end
end
endmodule
`default_nettype wire
